package ru.dbotthepony.kstarbound.client.world.`object`

import com.google.common.collect.ImmutableMap
import com.google.gson.JsonObject
import ru.dbotthepony.kstarbound.RegistryObject
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.client.world.ClientWorld
import ru.dbotthepony.kstarbound.defs.Drawable
import ru.dbotthepony.kstarbound.defs.animation.AnimationDefinition
import ru.dbotthepony.kstarbound.defs.image.ImageReference
import ru.dbotthepony.kstarbound.defs.`object`.ObjectDefinition
import ru.dbotthepony.kstarbound.world.`object`.WorldObject
import ru.dbotthepony.kvector.vector.Vector2i

class ClientWorldObject(world: ClientWorld, prototype: RegistryObject<ObjectDefinition>, pos: Vector2i) : WorldObject(world, prototype, pos) {
	val clientWorld get() = world as ClientWorld

	var animationParts: ImmutableMap<String, ImageReference> by Property()
	var imagePosition: Vector2i by Property(Vector2i.ZERO)
	var animationPosition: Vector2i by Property(Vector2i.ZERO)

	val animation: AnimationDefinition? by LazyData(listOf("animationCustom", "animation")) {
		/*
		val custom = dataValue("animationCustom")

		if (custom !is JsonObject) {
			prototype.value.animation?.value
		} else if (prototype.value.animation != null) {
			animAdapter.fromJsonTree(merge(prototype.value.animation.json!!, custom))
		} else {
			null
		}*/

		null
	}

	val drawables: List<Drawable> by LazyData {
		if (orientationIndex !in 0 until validOrientations) {
			return@LazyData listOf()
		} else {
			return@LazyData orientations[orientationIndex].drawables
		}
	}

	companion object {
		private val animAdapter by lazy {
			Starbound.gson.getAdapter(AnimationDefinition::class.java)
		}

		fun create(world: ClientWorld, data: JsonObject): ClientWorldObject {
			return create(world, data, ::ClientWorldObject)
		}
	}
}
